#define MCU_CLK 1000000

#include <inttypes.h>
#include <stdlib.h>
#include <string.h>

#include "spin_delay.h"
#include "lcd_pcd8544.h"
#include "lcd_pcd8544_text.h"
#include "tinymenu.h"
#include "joystick.h"


#if STRINGS_IN_FLASH
void my_select(void *arg, const char *name);
const char _select_1[MENU_ENTRY_NAMELEN] PROGMEM = "select 1";
const char _select_2[MENU_ENTRY_NAMELEN] PROGMEM = "select 2";
const char _select_3[MENU_ENTRY_NAMELEN] PROGMEM = "select 3";
const char _select_4[MENU_ENTRY_NAMELEN] PROGMEM = "select 4";
const char _select_5[MENU_ENTRY_NAMELEN] PROGMEM = "select 5";
const char _select_6[MENU_ENTRY_NAMELEN] PROGMEM = "select 6";
const char _select_7[MENU_ENTRY_NAMELEN] PROGMEM = "select 7";
const char _submenu_1[MENU_ENTRY_NAMELEN] PROGMEM = "submenu 1";
const char _submenu_2[MENU_ENTRY_NAMELEN] PROGMEM = "submenu 2";
#define select_1 _select_1
#define select_2 _select_2
#define select_3 _select_3
#define select_4 _select_4
#define select_5 _select_5
#define select_6 _select_6
#define select_7 _select_7
#define submenu_1 _submenu_1
#define submenu_2 _submenu_2
#else
void my_select(void *arg, char *name);
#define select_1 "select 1"
#define select_2 "select 2"
#define select_3 "select 3"
#define select_4 "select 4"
#define select_5 "select 5"
#define select_6 "select 6"
#define select_7 "select 7"
#define submenu_1 "submenu 1"
#define submenu_2 "submenu 2"
#endif

extern menu_t main_menu; // ### Empty forward declaration

menu_t sub1_menu = {
	.top_entry = 0,
	.current_entry = 0,
	.entry =  {
		{.flags = 0,
		 .select = my_select,
		 .name = select_2,   // ###
		 .value = NULL,      // ###
		},
		{.flags = 0,
		 .select = my_select,
		 .name = select_3,   // ###
		 .value = NULL,      // ###
		},
		{.flags = 0,
		 .select = my_select,
		 .name = select_4,   // ###
		 .value = NULL,      // ###
		},
	},
	.num_entries = 3,
	.previous = &main_menu, // NULL,
};

menu_t sub2_menu = {
	.top_entry = 0,
	.current_entry = 0,
	.entry = {
		{.flags = 0,
		 .select = my_select,
		 .name = select_5,   // ###
		 .value = NULL,      // ###
		},
		{.flags = 0,
		 .select = my_select,
		 .name = select_6,   // ###
		 .value = NULL,      // ###
		},
		{.flags = 0,
		 .select = my_select,
		 .name = select_7,   // ###
		 .value = NULL,
		}
	},
	.num_entries = 3,
	.previous = &main_menu, // NULL,
};

menu_t main_menu = {
	.top_entry = 0,
	.current_entry = 0,
	.entry = {
		{.flags = MENU_FLAG_SUBMENU,
		 .select = NULL,
		 .name = submenu_1,   // ###
		 .value = &sub1_menu, // ###
		},
		{.flags = MENU_FLAG_SUBMENU,
		 .select = NULL,
		 .name = submenu_2,   // ###
		 .value = &sub2_menu, // ###
		},
		{.flags = 0,
		 .select = my_select, // ###
		 .name = select_1,    // ###
		 .value = NULL,       // ###
		}
	},
	.num_entries = 3,
	.previous = NULL,
};

menu_context_t menu_context = {
	.x_loc = 0,
	.y_loc = 0,
	.height = 2,
	.width = 14,
};

#if STRINGS_IN_FLASH	
void my_select(void *arg, PGM_P name)
{
  const char *p = name;
	lcd_set_pos(0,20);
  while(pgm_read_byte(p))
    lcd_putchar(pgm_read_byte(p++));
}
#else
void my_select(void *arg, char *name)
{
	lcd_set_pos(0,20);
	lcd_puts(name);
}
#endif

int main()
{
	uint8_t j;

#if STRINGS_IN_FLASH
#if STRINGS_IN_FLASH_DEBUG
  {
    menu_entry_t *p;
    char buffer[30];
    p = &(main_menu.entry[0]); // Watch in AVR Studio cann't show p->name (ERRC0001: Unspecified type)
    strcpy_P(buffer, p->name); // but access is OK!
    p = &(main_menu.entry[0]); // Dummy
  }
#endif
#endif
	
	joystick_init();
	
  //menu_enter(&menu_context, NULL);
	menu_enter(&menu_context, &main_menu);  // ###
	
	while (1) {
		ms_spin(50);
		j = joystick_read();
	
		switch(j) {                           // ### BREALPOINT 2 ###
			case JOYSTICK_UP:
				lcd_clear();
				menu_next_entry(&menu_context);
				break;
			case JOYSTICK_DOWN:
				menu_prev_entry(&menu_context);
				break;
			case JOYSTICK_LEFT:
				menu_exit(&menu_context);
				break;
			case JOYSTICK_RIGHT:
			case JOYSTICK_BUTTON:
				menu_select(&menu_context);
				break;
		}
	}
}
